package com.exparrot.pete.stats;


import java.util.*;

public class City {

	public int possiblenumberofsites = 100;

	public int months = 12 * 6; //Do analysis for six years


	//Accident rates taken from 
	// http://www.cambridgeshire.gov.uk/sub/eandt/rdsafety/chap7.pdf
	//Over 30 sites in 3 years

	public double fatal = 7.3 / 30 / 36;
	public double serious = 45.7 / 30 / 36;
	public double slight = 206.0 / 30 / 36;


	public double fatalweight = 41.46;
	public double seriousweight = 5.58;
	public double slightweight = 1;
	
	public int accidentdata[][][];

	private ArrayList cameras;
	
//	public int cameras = 30;

	public City() {
		
//		System.out.println("Fatal per month " + fatal + " Serious " +
//		serious);
		accidentdata = new int[possiblenumberofsites][months][3];

		//Generate some accident statistics for each site
		for (int i = 0; i < possiblenumberofsites; i++) {

			fillSite(i);
		}
		
		cameras = new ArrayList();
	
	}

	public void fillSite(int site) {

		//Add a random site weighting
		//double weighting = java.lang.Math.random() * 2;
		double weighting = 1;

		//Fatal
		Poisson p = new Poisson(fatal * weighting);
		for (int i = 0; i < months; i++) {
			accidentdata[site][i][0] = p.getSample();
		}

		//Serious
		p = new Poisson(serious * weighting);
		for (int i = 0; i < months; i++) {
			accidentdata[site][i][1] = p.getSample();
/*			System.out.println("Site " + i + " has " +
			accidentdata[site][i][1] + " serious accidents");*/
		}

		//Slight
//		System.out.println("Rate " + (slight * weighting));
		p = new Poisson(slight * weighting);
		for (int i = 0; i < months; i++) {
			accidentdata[site][i][2] = p.getSample();
		}


		
	}

	//This method identifies the sites that should recieve cameras
	public void identify() {
	
		
		for (int i = 0; i < possiblenumberofsites; i++) {
			int accidents = 0;
			for (int j = 0; j < 36; j++) {
				
				accidents += accidentdata[i][j][0] + accidentdata[i][j][1];
				
			}
			
			if (accidents > 3) {
				cameras.add(new java.lang.Integer(i));

			}
		}

		

	}

	/** A three year analysis as done like Cambridge City council */
	public double threeyearanalysis(boolean display) {

		if (display) System.out.println("Conducting three year study");
		int priortotal = 0;
		int aftertotal = 0;

		for (int i = 0; i < cameras.size(); i++) {
			int site = ((Integer)cameras.get(i)).intValue();
			
			int accidents = 0;
			for (int j = 0; j < 36; j++) {
				
				accidents += accidentdata[site][j][0] + accidentdata[site][j][1];
				
			}


			int afteraccidents = 0;

			for (int j = 36; j < 72; j++) {
				afteraccidents += accidentdata[site][j][0] + accidentdata[site][j][1];
			}

			if (display) {
			System.out.println("Site " + site + "; Improvement in KSI " + 	accidents + " to "
			+ afteraccidents + " = " + (((double)accidents -
			(double)afteraccidents) / (double)accidents * (double)100)
			+ " %");
			}
			
			priortotal += accidents;
			aftertotal += afteraccidents;
		}

		double percentage = + ((double)priortotal -
		(double)aftertotal)/((double)priortotal) * (double)100;

		if (display) {
			System.out.println("Overall sites " + priortotal + " to " +
			aftertotal);
			System.out.println("Percentage " + percentage);
		}
		
		return percentage;

	}

	/** A One year analysis done as Stephanie Hess' paper */
	public double oneyearanalysis(boolean display) {

		if (display) System.out.println("Conducting one year study");
		double priortotal = 0;
		double aftertotal = 0;

		for (int i = 0; i < cameras.size(); i++) {
			int site = ((Integer)cameras.get(i)).intValue();
			
			double accidents = 0;
			for (int j = 24; j < 36; j++) {
				
				accidents += (fatalweight * (double)accidentdata[site][j][0])
				+ (seriousweight * (double)accidentdata[site][j][1])
				+ (slightweight * (double)accidentdata[site][j][2]);
				
			}


			double afteraccidents = 0;

			for (int j = 36; j < 48; j++) {
				afteraccidents += (fatalweight * (double)accidentdata[site][j][0])
				+ (seriousweight * (double)accidentdata[site][j][1])
				+ (slightweight * (double)accidentdata[site][j][2]);
			}

			if (display) {
				System.out.println("Site " + site + 
				"; Improvement in weighted value " + 	accidents + " to "
				+ afteraccidents + " = " + (((double)accidents -
				(double)afteraccidents) / (double)accidents * (double)100)
				+ " %");
			}
			priortotal += accidents;
			aftertotal += afteraccidents;
		}
		double percentage = ((double)priortotal -
		(double)aftertotal)/((double)priortotal) * (double)100;
		
		if (display) {
			System.out.println("Overall sites " + priortotal + " to " +
			aftertotal);
			System.out.println("Percentage " + percentage);
		}

		return percentage;
		

	}

	
	public static void main (String[] args){
		City c = new City();
		c.identify();
		
		double cc = c.threeyearanalysis(true);
		double sh = c.oneyearanalysis(true);
		System.out.println("\n\n\n");
		System.out.println("Presenting Study Results");
		System.out.println(c.possiblenumberofsites + " sites considered");
		System.out.println(c.cameras.size() + " Toilets installed");
		System.out.println("City Council " + cc + "% improvement");
		System.out.println("StephanieHess " + sh + "% improvement");
	}

}
